@@ -1,6 +1,8 @@
 /* 
- * yppasswdd
- * Copyright 1994 Olaf Kirch, <okir@monad.swb.de>
+ * update.c	Common update routines for normal passwd file and
+ *		J.F. Haugh's shadow password suite.
+ *
+ * Copyright 1994, 1995 Olaf Kirch, <okir@monad.swb.de>
  *
  * This program is covered by the GNU General Public License, version 2.
  * It is provided in the hope that it is useful. However, the author
@@ -67,13 +69,18 @@ void reaper( int sig );
 static int
 validate_string(char *str)
 {
-    while (*str && !iscntrl(*str)) str++;
+    while (*str && *str != ':' && !iscntrl(*str)) str++;
     return (*str == '\0');
 }
 
 static int
 validate_args(struct xpasswd *pw)
 {
+    if (pw->pw_name[0] == '-' || pw->pw_name[0] == '+') {
+	syslog(LOG_ALERT, "attempt to modify NIS passwd entry \"%s\"",
+			pw->pw_name);
+    }
+
     return validate_string(pw->pw_passwd)
        &&  validate_string(pw->pw_shell)
        &&  validate_string(pw->pw_gecos);
@@ -157,7 +164,8 @@ yppasswdproc_pwupdate_1(yppasswd *yppw, struct svc_req *rqstp)
 
             /* Check the password.
              */
-            if (strcmp(crypt(yppw->oldpass, pw->pw_passwd), pw->pw_passwd)) {
+            if (pw->pw_passwd[0] != '\0' &&
+		strcmp(crypt(yppw->oldpass, pw->pw_passwd), pw->pw_passwd)) {
         	syslog ( LOG_WARNING, "%s rejected", logbuf );
                 syslog ( LOG_WARNING, "Invalid password." );
                 sleep(1);
@@ -309,7 +317,8 @@ yppasswdproc_spwupdate_1(yppasswd *yppw, struct svc_req *rqstp)
     /*
      * Check the password.
      */
-    if (strcmp(crypt(yppw->oldpass, spw->sp_pwdp), spw->sp_pwdp)) {
+    if (spw->sp_pwdp[0] != '\0' &&
+        strcmp(crypt(yppw->oldpass, spw->sp_pwdp), spw->sp_pwdp)) {
 	syslog ( LOG_WARNING, "%s rejected", logbuf );
 	syslog ( LOG_WARNING, "Invalid password." );
 	spw_close(); spw_unlock();
